# coding: utf-8

"""
Copyright 2016 SmartBear Software

    Licensed under the Apache License, Version 2.0 (the "License");
    you may not use this file except in compliance with the License.
    You may obtain a copy of the License at

        http://www.apache.org/licenses/LICENSE-2.0

    Unless required by applicable law or agreed to in writing, software
    distributed under the License is distributed on an "AS IS" BASIS,
    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
    See the License for the specific language governing permissions and
    limitations under the License.

    Ref: https://github.com/swagger-api/swagger-codegen
"""

from datetime import datetime
from datetime import date
from pprint import pformat
from six import iteritems
import re
import json

from ..utils import sanitize_for_serialization

# type hinting support
from typing import TYPE_CHECKING
from typing import List
from typing import Dict

if TYPE_CHECKING:
    from . import RoutePathResponse
    from . import ServiceGoalTemplateReference
    from . import WfmVersionedEntityMetadata

class PlanningGroup(object):
    """
    NOTE: This class is auto generated by the swagger code generator program.
    Do not edit the class manually.
    """
    def __init__(self) -> None:
        """
        PlanningGroup - a model defined in Swagger

        :param dict swaggerTypes: The key is attribute name
                                  and the value is attribute type.
        :param dict attributeMap: The key is attribute name
                                  and the value is json key in definition.
        """
        self.swagger_types = {
            'id': 'str',
            'name': 'str',
            'service_goal_template': 'ServiceGoalTemplateReference',
            'route_paths': 'list[RoutePathResponse]',
            'metadata': 'WfmVersionedEntityMetadata',
            'self_uri': 'str'
        }

        self.attribute_map = {
            'id': 'id',
            'name': 'name',
            'service_goal_template': 'serviceGoalTemplate',
            'route_paths': 'routePaths',
            'metadata': 'metadata',
            'self_uri': 'selfUri'
        }

        self._id = None
        self._name = None
        self._service_goal_template = None
        self._route_paths = None
        self._metadata = None
        self._self_uri = None

    @property
    def id(self) -> str:
        """
        Gets the id of this PlanningGroup.
        The globally unique identifier for the object.

        :return: The id of this PlanningGroup.
        :rtype: str
        """
        return self._id

    @id.setter
    def id(self, id: str) -> None:
        """
        Sets the id of this PlanningGroup.
        The globally unique identifier for the object.

        :param id: The id of this PlanningGroup.
        :type: str
        """
        

        self._id = id

    @property
    def name(self) -> str:
        """
        Gets the name of this PlanningGroup.


        :return: The name of this PlanningGroup.
        :rtype: str
        """
        return self._name

    @name.setter
    def name(self, name: str) -> None:
        """
        Sets the name of this PlanningGroup.


        :param name: The name of this PlanningGroup.
        :type: str
        """
        

        self._name = name

    @property
    def service_goal_template(self) -> 'ServiceGoalTemplateReference':
        """
        Gets the service_goal_template of this PlanningGroup.
        The ID of the service goal template associated with this planning group

        :return: The service_goal_template of this PlanningGroup.
        :rtype: ServiceGoalTemplateReference
        """
        return self._service_goal_template

    @service_goal_template.setter
    def service_goal_template(self, service_goal_template: 'ServiceGoalTemplateReference') -> None:
        """
        Sets the service_goal_template of this PlanningGroup.
        The ID of the service goal template associated with this planning group

        :param service_goal_template: The service_goal_template of this PlanningGroup.
        :type: ServiceGoalTemplateReference
        """
        

        self._service_goal_template = service_goal_template

    @property
    def route_paths(self) -> List['RoutePathResponse']:
        """
        Gets the route_paths of this PlanningGroup.
        Set of route paths associated with the planning group

        :return: The route_paths of this PlanningGroup.
        :rtype: list[RoutePathResponse]
        """
        return self._route_paths

    @route_paths.setter
    def route_paths(self, route_paths: List['RoutePathResponse']) -> None:
        """
        Sets the route_paths of this PlanningGroup.
        Set of route paths associated with the planning group

        :param route_paths: The route_paths of this PlanningGroup.
        :type: list[RoutePathResponse]
        """
        

        self._route_paths = route_paths

    @property
    def metadata(self) -> 'WfmVersionedEntityMetadata':
        """
        Gets the metadata of this PlanningGroup.
        Version metadata for the planning group

        :return: The metadata of this PlanningGroup.
        :rtype: WfmVersionedEntityMetadata
        """
        return self._metadata

    @metadata.setter
    def metadata(self, metadata: 'WfmVersionedEntityMetadata') -> None:
        """
        Sets the metadata of this PlanningGroup.
        Version metadata for the planning group

        :param metadata: The metadata of this PlanningGroup.
        :type: WfmVersionedEntityMetadata
        """
        

        self._metadata = metadata

    @property
    def self_uri(self) -> str:
        """
        Gets the self_uri of this PlanningGroup.
        The URI for this object

        :return: The self_uri of this PlanningGroup.
        :rtype: str
        """
        return self._self_uri

    @self_uri.setter
    def self_uri(self, self_uri: str) -> None:
        """
        Sets the self_uri of this PlanningGroup.
        The URI for this object

        :param self_uri: The self_uri of this PlanningGroup.
        :type: str
        """
        

        self._self_uri = self_uri

    def to_dict(self):
        """
        Returns the model properties as a dict
        """
        result = {}

        for attr, _ in iteritems(self.swagger_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value

        return result

    def to_json(self):
        """
        Returns the model as raw JSON
        """
        return json.dumps(sanitize_for_serialization(self.to_dict()))

    def to_str(self):
        """
        Returns the string representation of the model
        """
        return pformat(self.to_dict())

    def __repr__(self):
        """
        For `print` and `pprint`
        """
        return self.to_str()

    def __eq__(self, other):
        """
        Returns true if both objects are equal
        """
        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        """
        Returns true if both objects are not equal
        """
        return not self == other

